<?php

class Smartcart_Helpers
{

    private $wpdb;
    private $log;

    public function __construct()
    {
        $this->log = new Smartcart_Logs();
        global $wpdb;
        $this->wpdb = $wpdb;
    }

    public function verifySecretKey($request)
    {
        $requestData = $request->get_params();
        $secret = $request->get_header("X-Smartcat-Secret");
        $verifyHash = hash_hmac('sha256', json_encode($requestData), get_option('smartcat_wpml_secret'));
        return $secret === $verifyHash;
    }

    public function getOption($key)
    {
        return $this->wpdb->get_col(
            $this->wpdb->prepare(
                "SELECT value FROM {$this->wpdb->prefix}smartcat_options WHERE name = %d",
                [$key]
            )
        )[0];
    }

    public function fetchTranslatedPosts($sourceDocumentsIds, $itemsCount, $documentId = NULL)
    {
        $notInScheme = implode(', ', array_fill(0, count($sourceDocumentsIds), '%d'));
        $args = array_merge(["post", "page", "trash"], $sourceDocumentsIds, $documentId ? [$documentId] : [],
            [$itemsCount]);
        $whereId = $documentId ? 'AND ID > %d' : '';
        $query = "SELECT * FROM {$this->wpdb->prefix}posts WHERE (post_type = %s OR post_type = %s) AND post_status != %s AND ID NOT IN ({$notInScheme}) {$whereId} ORDER BY ID ASC LIMIT %d";

        try {
            return $this->wpdb->get_results(
                $this->wpdb->prepare($query, $args)
            );
        } catch (\Exception $e) {
            $this->log->add("Critical", "Error select translated documents", $e->getTraceAsString());
        }
    }

    public function getLastPostModified()
    {
        return $this->wpdb->get_col(
            $this->wpdb->prepare(
                "SELECT post_modified FROM " . $this->wpdb->prefix . "posts WHERE post_type = %s AND post_status = %s ORDER BY post_modified DESC LIMIT 1",
                ["post", "publish"]
            )
        )[0];
    }
    
    public function getCategory($postId)
    {
        return (int)$this->wpdb->get_col(
            $this->wpdb->prepare(
                "SELECT term_taxonomy_id FROM "
                . $this->wpdb->prefix .
                "term_relationships WHERE object_id = %d",
                [$postId]
            )
        )[0] ?? NULL;
    }

    public function updateWpmlTranslations(int $documentId, int $originalId, string $targetLang, $type = "post", string $sourceLang = NULL): void
    {
        global $sitepress;
        $wpmlTr = $this->getWpmlTr($originalId, $sourceLang);
        $this->wpdb->update(
            $this->wpdb->prefix . "icl_translations",
            [
                "element_type" => "post_$type",
                "trid" => $wpmlTr,
                "language_code" => $targetLang,
                "source_language_code" => $sourceLang ?? $sitepress->get_default_language()
            ],
            [
                "element_id" => $documentId
            ]
        );
    }

    public function getWpmlTr($originalId, $sourceLang = NULL)
    {
        global $sitepress;
        return $this->wpdb->get_col(
            $this->wpdb->prepare(
                "SELECT trid from {$this->wpdb->prefix}icl_translations WHERE element_id = %d AND language_code = %s LIMIT %d",
                [$originalId, $sourceLang ?? $sitepress->get_default_language(), 1]
            )
        )[0] ?? NULL;
    }

    public function checkWpmlTranslations(int $originalId, string $targetLang, $sourceLang = NULL)
    {
        return $this->wpdb->get_row(
            $this->wpdb->prepare(
                "SELECT * FROM {$this->wpdb->prefix}icl_translations WHERE language_code = %s AND trid = %d",
                [$targetLang, $this->getWpmlTr($originalId, $sourceLang)]
            )
        );
    }

    public function fetchIclTranslation(int $postId)
    {
        return $this->wpdb->get_row(
            $this->wpdb->prepare(
                "SELECT * FROM {$this->wpdb->prefix}icl_translations WHERE element_id = %d",
                [$postId]
            )
        );
    }

    public function fetchIclTranslationOriginal(int $trId, $sourceLang)
    {
        global $sitepress;

        return $this->wpdb->get_row(
            $this->wpdb->prepare(
                "SELECT * FROM {$this->wpdb->prefix}icl_translations WHERE trid = %d AND language_code = %s",
                [$trId, $sourceLang ?? $sitepress->get_default_language()]
            )
        );
    }

    public function sourceDocumentsIds(string $type, string $defaultLang): array
    {
        try {

        } catch (\Exception $e) {
            $this->log->add("Error", "Error select not source document", $e->getTraceAsString());
        }

        $sourceDocuments = $this->wpdb->get_col(
            $this->wpdb->prepare(
                "SELECT element_id FROM {$this->wpdb->prefix}icl_translations WHERE (element_type = %s OR element_type = %s) AND language_code = %s",
                ["post_post", "post_page", $defaultLang]
            )
        );

        return $sourceDocuments;
    }

    public function responseErr(string $code, string $message = null): array
    {
        return [
            "code" => $code,
            "message" => $message,
            "data" => [
                "status" => "failed"
            ]
        ];
    }

    /**
     * @throws ErrorException
     */
    public function fetchNotSourceDocuments($defaultLang)
    {
        try {
            $list = $this->wpdb->get_col(
                $this->wpdb->prepare(
                    "SELECT element_id FROM {$this->wpdb->prefix}icl_translations WHERE (element_type = %s OR element_type = %s) AND language_code != %s",
                    ["post_post", "post_page", $defaultLang]
                )
            );
        } catch (\ErrorException $errorException) {
            throw $errorException;
        }

        return $list;
    }

    public function fetchDocuments($notSourceDocuments, $itemsCount, $documentId = NULL, $modificationDate = NULL)
    {
        $params = ["post", "page", "publish"];

        $whereDocumentId = $documentId ? "AND ID > %d" : "";
        $whereDate = $modificationDate ? "AND post_modified > %s" : "";

        $notIn = implode(', ', array_fill(0, count($notSourceDocuments), '%d'));

        $otherArgs = [];

        if ($whereDocumentId !== "") {
            $otherArgs[] = $documentId;
        }

        if ($whereDate !== "") {
            $otherArgs[] = $modificationDate;
        }

        $otherArgs[] = $itemsCount;

        $args = array_merge($params, $notSourceDocuments, $otherArgs);

        $prepare = $this->wpdb->prepare(
            "SELECT * FROM {$this->wpdb->prefix}posts WHERE (post_type = %s OR post_type = %s) AND post_status = %s AND ID NOT IN ($notIn) $whereDocumentId $whereDate ORDER BY ID ASC LIMIT %d", $args
        );

        try {
            $list = $this->wpdb->get_results($prepare);
        } catch (\Exception $exception) {
            throw $exception;
        }

        return $list;
    }

    /**
     * @throws ErrorException
     */
    public function intOrNull($int, $param = NULL)
    {
        if (is_int($int) || empty($int)) {
            return $int;
        }
        throw new ErrorException("Parameter id is not NULL and Integer type, param: $param", 500);
    }
}