<?php

class Smartcart_Import
{

    private $wpdb;
    private $disableSecretVerification = false;

    private $helpers;
    private $log;
    private $acfHelper;

    private $items_count = 5;
    private $sourceLang = NULL;

    public function __construct()
    {
        $this->helpers = new Smartcart_Helpers;
        $this->log = new Smartcart_Logs();
        $this->acfHelper = new Smartcart_Acf_Helper();

        global $wpdb;
        $this->wpdb = $wpdb;
    }

    /**
     * @throws ErrorException
     */
    public function request($request)
    {
        $this->validateRequest($request);

        $requestData = $this->requestData($request);
        $this->sourceLang = $requestData->sourceLang ?? NULL;

        $categories = $this->fetchCategories(
            $this->fetchTranslatedCategoriesIds($requestData->targetLanguages)
        );

        $posts = [];
        $isImport = false;

        if (isset($requestData->ids) && !empty($requestData->ids)) {
            $isImport = true;
            foreach ($requestData->ids as $id) {
                $trid = apply_filters( 'wpml_element_trid', NULL, $id);
                $translations = apply_filters( 'wpml_get_element_translations', NULL, $trid);
                foreach ($translations as $translation) {
                    if (in_array($translation->language_code, $requestData->targetLanguages)) {
                        $posts[] = get_post($translation->element_id);
                    }
                }
            }
        } else {
            $posts = $this->fetchPosts(
                $this->fetchTranslatedPostsIds($requestData->targetLanguages),
                $requestData->documentId
            );
        }

        return $this->response($posts, $categories, $isImport);
    }

    /**
     * @throws ErrorException
     */
    private function validateRequest($request)
    {
        if (!function_exists('icl_object_id')) {
            throw new ErrorException("WPML plugin is not installed or activated. SmartCat Connector is not active", 404);
        }

        if (!$this->helpers->verifySecretKey($request) && $this->disableSecretVerification === false) {
            throw new ErrorException("Error while authorizing a request by secret key", 401);
        }
    }

    private function requestData($request)
    {
        $requestData = $request->get_params();
        return (object)[
            "documentId" => $requestData["documentId"] ?? NULL,
            "sourceLang" => $requestData["sourceLang"] ?? NULL,
            "targetLanguages" => $requestData["targetLanguages"] ?? [],
            "ids" => $requestData["ids"] ?? NULL
        ];
    }

    private function fetchTranslatedPostsIds($targetLanguages)
    {
        $inString = implode(', ', array_fill(0, count($targetLanguages), '%s'));
        $query = "SELECT element_id FROM {$this->wpdb->prefix}icl_translations WHERE (element_type = %s OR element_type = %s) AND language_code IN ($inString)";
        $args = array_merge(["post_post", "post_page"], $targetLanguages);
        return $this->wpdb->get_col(
            $this->wpdb->prepare($query, $args)
        );
    }

    private function fetchTranslatedCategoriesIds($targetLanguages)
    {
        $inString = implode(', ', array_fill(0, count($targetLanguages), '%s'));
        $query = "SELECT element_id,language_code FROM {$this->wpdb->prefix}icl_translations WHERE element_type = %s AND language_code IN ($inString)";
        $args = array_merge(["tax_category"], $targetLanguages);
        return $this->wpdb->get_results(
            $this->wpdb->prepare($query, $args)
        );
    }

    /**
     * @throws Exception
     */
    private function fetchPosts($ids, $documentId)
    {

        $whereInScheme = implode(', ', array_fill(0, count($ids), '%d'));

        $args = array_merge(
            ["post", "page", "trash"],
            $ids,
            $documentId ? [$documentId, $this->items_count] : [$this->items_count]
        );

        $whereId = $documentId ? 'AND ID > %d' : '';
        $query = "SELECT * FROM {$this->wpdb->prefix}posts WHERE (post_type = %s OR post_type = %s) AND post_status != %s AND ID IN ({$whereInScheme}) {$whereId} ORDER BY ID ASC LIMIT %d";

        try {
            return $this->wpdb->get_results(
                $this->wpdb->prepare($query, $args)
            );
        } catch (\Exception $e) {
            $this->log->add("Critical", "Error select translated documents", $e->getTraceAsString());
            throw $e;
        }
    }

    private function fetchCategories(array $categories)
    {

        $categoriesList = [];

        foreach ($categories as $categoryRow) {
            $id = $categoryRow->element_id;
            $category = $this->fetchTerm($id);

            if (is_null($category)) {
                continue;
            }

            $categoriesList[] = [
                "id" => $category->term_id,
                "title" => $category->name,
                "description" => $this->getCategoryDescription($id),
                "parent" => $this->getCategoryParent($id),
                "lang" => $categoryRow->language_code
            ];
        }

        return $categoriesList;
    }

    private function getOriginalPostSlug($postId)
    {
        $post = get_post($postId);
        $permalink = get_permalink($postId);
        $linkComponents = parse_url($permalink);
        $uri = $linkComponents['path'] ?? NULL;
        if (empty($uri)) {
            throw new \Exception('Key "path" not found');
        }
        $uri = trim($uri, '/');
        $explodeUri = explode('/', $uri);
        if (count($explodeUri) > 1) {
            array_pop($explodeUri);
            $postName = str_replace('/', '-', implode('-', $explodeUri));
            $postName .= '--' . $post->post_name;
        } else {
            $postName = str_replace('/', '-', $uri);
        }
        return $postName;
    }

    private function normalizePosts($posts)
    {
        $normalizePosts = [];
        foreach ($posts as $post) {
            $postType = "post_{$post->post_type}";

            $trid = $this->getTrid($post->ID, $postType);
            if (is_null($trid)) {
                continue;
            }

            $originalId = $this->getOriginalId($trid, $postType);
            if (is_null($originalId)) {
                continue;
            }

            $entity = [
                "id" => $originalId,
                "title" => $post->post_title,
                "content" => $post->post_content,
                "postName" => urldecode($this->getOriginalPostSlug($originalId)),
                "slug" => $post->post_name,
                "type" => $post->post_type,
                "category" => $this->helpers->getCategory($post->ID),
                "lang" => $this->getLanguage($post->ID, $post->post_type)
            ];

            if ((bool)get_option("smartcat_wpml_acf_using")) {
                $entity["acf"] = $this->acfHelper->customFields($post->ID);
            }

            $normalizePosts[] = $entity;
        }

        return $normalizePosts;
    }

    private function getOriginalId($trid, $postType)
    {
        $translations = apply_filters('wpml_get_element_translations', NULL, $trid, $postType);
        foreach ($translations as $t) {
            if ($t->language_code === $this->sourceLang) {
                return $t->element_id;
            }
        }
        return NULL;
    }

    private function getTrid($postId, $postType)
    {
        return apply_filters('wpml_element_trid', NULL, $postId, $postType);
    }

    private function getLanguage($postId, $postType)
    {
        $args = ['element_id' => $postId, 'element_type' => $postType];
        return apply_filters('wpml_element_language_code', null, $args);
    }

    private function fetchTerm($id)
    {
        $query = "SELECT * FROM {$this->wpdb->terms} WHERE term_id = %d";
        $args = [$id];

        return $this->wpdb->get_row(
            $this->wpdb->prepare($query, $args)
        );
    }

    private function getCategoryDescription($id)
    {
        return $this->wpdb->get_col(
            $this->wpdb->prepare(
                "SELECT description FROM {$this->wpdb->term_taxonomy} WHERE term_id = %d",
                [$id]
            )
        )[0];
    }

    private function getCategoryParent($id)
    {
        return $this->wpdb->get_col(
            $this->wpdb->prepare(
                "SELECT parent FROM {$this->wpdb->term_taxonomy} WHERE term_id = %d",
                [$id]
            )
        )[0];
    }

    private function getBatchKey($posts)
    {
        $lastDocument = array_pop($posts);

        return base64_encode(
            json_encode([
                "documentId" => $lastDocument->ID,
                "lastModification" => $lastDocument->post_modified
            ])
        );
    }

    private function response($posts, $categories, $isImport)
    {
        return [
            "data" => [
                "status" => "success",
                "hash" => $isImport ? NULL : $this->getBatchKey($posts),
                "documents" => $this->normalizePosts($posts),
                "categories" => $categories
            ]
        ];
    }
}