<?php

class Smartcart_Pull
{

    private $wpdb;
    private $disableSecretVerification = false;

    private $helpers;
    private $log;
    private $acfHelper;
    private $requestData;

    private $items_count = 5;

    public function __construct()
    {
        $this->helpers = new Smartcart_Helpers;
        $this->log = new Smartcart_Logs();
        $this->acfHelper = new Smartcart_Acf_Helper();

        global $wpdb;
        $this->wpdb = $wpdb;
    }

    /**
     * @throws ErrorException
     * @throws Exception
     */
    public function request($request)
    {
        $this->validateRequest($request);

        $posts = NULL;
        $nextBatchKey = NULL;

        $this->requestData = $this->nextBatchParams($request);

        if (isset($this->requestData->ids) && !empty($this->requestData->ids)) {
            $posts = [];
            foreach ($this->requestData->ids as $id) {
                $posts[] = get_post($id);
            }
        } else {
            $notSourceDocuments = $this->helpers->fetchNotSourceDocuments($this->requestData->sourceLang);
            $notSourceDocuments = empty($notSourceDocuments) ? [0] : $notSourceDocuments;

            $posts = $this->helpers->fetchDocuments($notSourceDocuments, $this->items_count, $this->requestData->documentId, $this->requestData->date);
            $lastDocument = $posts[count($posts) - 1];

            $nextBatchKey = base64_encode(
                json_encode([
                    'documentId' => $lastDocument->ID,
                    'lastModification' => $this->helpers->getLastPostModified()
                ])
            );
        }

        return [
            'data' => [
                'status' => 'success',
                'hash' => $nextBatchKey,
                'documents' => $this->normalizePosts($posts)
            ]
        ];
    }

    /**
     * @throws ErrorException
     */
    private function validateRequest($request)
    {
        if (!function_exists('icl_object_id')) {
            throw new ErrorException('WPML plugin is not installed or activated. SmartCat Connector is not active', 404);
        }

        if (!$this->helpers->verifySecretKey($request) && $this->disableSecretVerification === false) {
            throw new ErrorException("Error while authorizing a request by secret key", 401);
        }

        if (is_null($this->nextBatchParams($request)->sourceLang)) {
            throw new ErrorException("Source lang not found", 500);
        }
    }

    private function nextBatchParams($request)
    {
        $requestData = $request->get_params();
        return (object)[
            'date' => $requestData['date'] ?? NULL,
            'sourceLang' => $requestData['sourceLang'] ?? NULL,
            'documentId' => $requestData['documentId'] ?? NULL,
            'ids' => $requestData['ids'] ?? NULL,
            'metaDataKeys' => $requestData['metaDataKeys'] ?? NULL,
        ];
    }

    /**
     * @throws Exception
     */
    private function normalizePosts(array $posts)
    {
        $normalizePosts = [];

        foreach ($posts as $post) {
            $json = new Json($this->getPostMetaData($post->ID));

            if ($post->ID == 1117) {
            }

            $entityArray = [
                'id' => $post->ID,
                'title' => $post->post_title,
                'postName' => urldecode($this->normalizePostName($post->ID, $post->post_name)),
                'slug' => $post->post_name,
                'content' => $post->post_content,
                'type' => $post->post_type,
                'category' => $this->helpers->getCategory($post->ID),
                'metadata' => $this->addContentFieldToArray(
                    empty($this->requestData->metaDataKeys)
                        ? $json->getJson(true)
                        : $json->extract($this->requestData->metaDataKeys)
                )
            ];

            if ((bool)get_option("smartcat_wpml_acf_using")) {
                $entityArray["acf"] = $this->acfHelper->customFields($post->ID);
            }

            $normalizePosts[] = $entityArray;
        }

        return $normalizePosts;
    }

    private function getPostMetaData($postId): array
    {
        $metaData = [];
        $postMeta = get_post_meta($postId);
        foreach ($postMeta as $key => $value) {
            $metaData[$key] = get_post_meta($postId, $key, true);
        }
        return $metaData;
    }

    private function normalizePostName($postId, $originalPostName): string
    {
        $permalink = get_permalink($postId);
        $linkComponents = parse_url($permalink);
        $uri = $linkComponents['path'] ?? NULL;
        if (empty($uri)) {
            throw new \Exception('Key "path" not found');
        }
        $uri = trim($uri, '/');
        $explodeUri = explode('/', $uri);
        if (count($explodeUri) > 1) {
            array_pop($explodeUri);
            $postName = str_replace('/', '-', implode('-', $explodeUri));
            $postName .= '--' . $originalPostName;
        } else {
            $postName = str_replace('/', '-', $uri);
        }
        return $postName;
    }

    private function addContentFieldToArray($data): array
    {
        return array_map(function ($item) {
            return ['content' => $item];
        }, $data);
    }
}