<?php

class Smartcart_Push
{

    const MORE_FIELDS_TYPES = ["repeater", "flexible_content", "group"];
    private $wpdb;
    private $disableSecretVerification = false;

    private $helpers;
    private $log;

    public function __construct()
    {
        $this->helpers = new Smartcart_Helpers;
        $this->log = new Smartcart_Logs();

        global $wpdb;
        $this->wpdb = $wpdb;
    }

    /**
     * @throws ErrorException
     */
    public function request($request)
    {
        $this->validateRequest($request);

        $sourceLang = $request->get_params()['sourceLang'] ?? NULL;
        $documents = $request->get_params()['documents'];

        foreach ($documents as $document) {

            // create correct assoc array for document
            $documentData = $this->prepareDocumentData($document);

            $originalId = $document['originalId'];
            $targetLang = $document['targetLang'];
            $this->log->add('Info', "Start adding document '$targetLang' for {$document["type"]} - $originalId");
            $type = $document['type'];

            try {
                $checkWpmlTranslations = $this->helpers->checkWpmlTranslations($originalId, $targetLang, $sourceLang);
            } catch (\Exception $e) {
                $this->log->add("Error", "Error select check wpml translation", $e->getTraceAsString());
            }

            if (is_null($checkWpmlTranslations)) {

                try {
                    $id = wp_insert_post($documentData);
                    $this->log->add("Info", "{$document["type"]} created $id");
                } catch (\Exception $e) {
                    $this->log->add("Critical", "Failed insert new post", $e->getTraceAsString());
                }

                $args = [
                    'element_id' => $id,
                    'language_code' => $targetLang,
                    'element_type' => "post_$type",
                    'source_language_code' => $sourceLang
                ];

                $trid = apply_filters('wpml_element_trid', NULL, $originalId, "post_$type");

                if (!is_null($trid)) {
                    $args['trid'] = apply_filters('wpml_element_trid', NULL, $originalId, "post_$type");
                }

                do_action('wpml_set_element_language_details', $args);

                // add Advanced Custom Fields into post
                if (!is_null($document["acf"]) && get_option('smartcat_wpml_acf_using')) {
                    $this->addAcf($id, $document["acf"], $document["originalId"]);
                }

                $this->updateMetaDataByOriginalPost($originalId, $id);

                if (!empty($document['metadata'])) {
                    $this->addMetaDataToPost($id, $document['metadata']);
                }

            } else {
                $id = $checkWpmlTranslations->element_id;
                $this->log->add("Info", "Updating {$document["type"]} $id");
                try {
                    wp_update_post([
                        'ID' => $id,
                        'post_title' => $document['title'],
                        'post_content' => $document['content']
                    ]);
                } catch (\Exception $e) {
                    $this->log->add('Critical', "Failed update post - $id", $e->getTraceAsString());
                }

                // add Advanced Custom Fields into post
                if (!isset($document['acf']) && get_option('smartcat_wpml_acf_using')) {
                    $this->addAcf($id, $document['acf'], $document['originalId']);
                }

                $this->updateMetaDataByOriginalPost($originalId, $id);

                if (!empty($document['metadata'])) {
                    $this->addMetaDataToPost($id, $document['metadata']);
                }
            }
        }

        return [
            'data' => [
                'status' => 'success'
            ]
        ];
    }

    private function addAcf($postId, $fields, $originalId = NULL)
    {
        foreach (get_fields($originalId) as $key => $value) {
            update_field($key, $value, $postId);
        }

        $this->log->add("Info", json_encode($fields, JSON_PRETTY_PRINT));

        foreach ($fields as $field) {
            $content = [];
            if (in_array($field["type"], self::MORE_FIELDS_TYPES)) {
                if (in_array($field["type"], ["repeater", "flexible_content"])) {
                    foreach (get_field($field["key"], $originalId) as $key => $item) {
                        foreach ($item as $k => $v) {
                            $content[$key][$k] = $field["content"][$key][$k] ?? $v;
                        }
                    }
                } else {
                    foreach (get_field($field["key"], $originalId) as $key => $item) {
                        $content[$key] = $field["content"][$key] ?? $item;
                    }
                }
            } else {
                $content = $field["content"];
            }

            $this->log->add("Info", json_encode($content, JSON_PRETTY_PRINT));

            update_field($field["key"], $content, $postId);
        }
    }

    private function addMetaDataToPost($postId, $metaData)
    {
        $postMetaData = $this->getPostMetaData($postId);
        $json = new Json($postMetaData);
        $jsonWithDots = $json->getJson(true);
        foreach ($metaData as $key => $value) {
            $jsonWithDots[$key] = $value['content'];
        }
        $undotNewMetaData = $json->undot($jsonWithDots);
        foreach ($undotNewMetaData as $key => $value) {
            update_post_meta($postId, $key, $value);
        }
    }

    private function updateMetaDataByOriginalPost($originalPostId, $targetPostId)
    {
        $originalMetaData = $this->getPostMetaData($originalPostId);
        foreach ($originalMetaData as $key => $value) {
            update_post_meta($targetPostId, $key, $value);
        }
    }

    private function getPostMetaData($postId): array
    {
        $metaData = [];
        $postMeta = get_post_meta($postId);
        foreach ($postMeta as $key => $value) {
            $metaData[$key] = get_post_meta($postId, $key, true);
        }
        return $metaData;
    }

    private function prepareDocumentData($document)
    {
        return [
            "post_title" => $document["title"],
            "post_content" => $document["content"],
            "post_category" => [$document["category"]],
            "post_type" => $document["type"],
            "post_status" => $document["postStatus"],
            "acf" => $document["acf"] ?? NULL
        ];
    }

    /**
     * @throws ErrorException
     */
    private function validateRequest($request)
    {
        if (!function_exists('icl_object_id')) {
            throw new ErrorException("WPML plugin is not installed or activated. SmartCat Connector is not active", 404);
        }

        if (!$this->helpers->verifySecretKey($request) && $this->disableSecretVerification === false) {
            throw new ErrorException("Error while authorizing a request by secret key", 401);
        }
    }
}