<?php

class Smartcat_Integration_Hub
{
    protected $loader;
    protected $plugin_name;
    protected $version;

    public function __construct()
    {
        if (defined('SMARTCAT_INTEGRATION_VERSION')) {
            $this->version = SMARTCAT_INTEGRATION_VERSION;
        } else {
            $this->version = '1.4.0';
        }
        $this->plugin_name = 'smartcat-wpml';

        $this->load_dependencies();
        $this->set_locale();
        $this->define_admin_hooks();
        $this->define_public_hooks();
        $this->migrateOptions();

    }

    private function migrateOptions()
    {
        global $wpdb;

        $table_smartcat_options = $wpdb->prefix . 'smartcat_options';
        if ($wpdb->get_var("SHOW TABLES LIKE '$table_smartcat_options'") === $table_smartcat_options) {
            $secret = $wpdb->get_col(
                $wpdb->prepare(
                    "SELECT value FROM {$table_smartcat_options} WHERE name = %d",
                    ["api_secret"]
                )
            )[0];

            if (!get_option("smartcat_wpml_secret")) {
                add_option("smartcat_wpml_secret", $secret);
            }

            if (!get_option("smartcat_wpml_acf_using")) {
                add_option("smartcat_wpml_acf_using", 0);
            }

            $wpdb->query("drop table {$table_smartcat_options}");
        }

        if ((bool)get_option("smartcat_wpml_acf_using") === true && !class_exists('ACF')) {
            update_option("smartcat_wpml_acf_using", 0);
        }

    }

    private function load_dependencies()
    {

        /**
         * The class responsible for orchestrating the actions and filters of the
         * core plugin.
         */
        require_once plugin_dir_path(dirname(__FILE__)) . 'includes/smartcat-wpml-loader.php';

        /**
         * The class responsible for defining internationalization functionality
         * of the plugin.
         */
        require_once plugin_dir_path(dirname(__FILE__)) . 'includes/smartcat-wpml-i18n.php';

        /**
         * The class responsible for defining all actions that occur in the admin area.
         */
        require_once plugin_dir_path(dirname(__FILE__)) . 'admin/smartcat-wpml-admin.php';

        /**
         * The class responsible for defining all actions that occur in the public-facing
         * side of the site.
         */
        require_once plugin_dir_path(dirname(__FILE__)) . 'public/smartcat-wpml-public.php';

        $this->loader = new Smartcat_Integration_Hub_Loader();

    }

    private function set_locale()
    {
        $plugin_i18n = new Smartcat_Integration_Hub_i18n();
        $this->loader->add_action('plugins_loaded', $plugin_i18n, 'load_plugin_textdomain');
    }

    private function define_admin_hooks()
    {

        $plugin_admin = new Smartcat_Integration_Hub_Admin($this->get_plugin_name(), $this->get_version());

        $this->loader->add_action('admin_enqueue_scripts', $plugin_admin, 'enqueue_styles');
        $this->loader->add_action('admin_enqueue_scripts', $plugin_admin, 'enqueue_scripts');

        $this->loader->add_action('admin_menu', $plugin_admin, 'add_plugin_admin_menu');
        $plugin_basename = plugin_basename(plugin_dir_path(__DIR__) . $this->plugin_name . '.php');
        $this->loader->add_filter('plugin_action_links_' . $plugin_basename, $plugin_admin, 'add_action_links');

        $this->loader->add_action("admin_post_smartcat_update_options", $plugin_admin, "smartcatUpdateOptions");
        $this->loader->add_action("admin_notices", $plugin_admin, "printPluginAdminNotices");

        if (!function_exists('icl_object_id')) {
            $this->loader->add_action('admin_notices', $plugin_admin, 'notExistWpmlNotice');
        }

        if (get_option("permalink_structure") === "") {
            $this->loader->add_action('admin_notices', $plugin_admin, 'notFriendlyUrlNotice');
        }

    }

    private function define_public_hooks()
    {
        $plugin_public = new Smartcat_Integration_Hub_Public($this->get_plugin_name(), $this->get_version());
        $this->loader->add_action('rest_api_init', $plugin_public, 'routes');
    }

    public function run()
    {
        $this->loader->run();
    }

    public function get_plugin_name()
    {
        return $this->plugin_name;
    }

    public function get_loader()
    {
        return $this->loader;
    }

    public function get_version()
    {
        return $this->version;
    }

}
